@@ -42,7 +42,7 @@
 /* cursor method - allocate a new cursor */
 
 #define psyco_conn_cursor_doc \
-"cursor(name=None, cursor_factory=extensions.cursor, withhold=None) -- new cursor\n\n"     \
+"cursor(name=None, cursor_factory=extensions.cursor, withhold=False) -- new cursor\n\n"     \
 "Return a new cursor.\n\nThe ``cursor_factory`` argument can be used to\n"  \
 "create non-standard cursors by passing a class different from the\n"       \
 "default. Note that the new class *should* be a sub-class of\n"             \
@@ -50,68 +50,81 @@
 ":rtype: `extensions.cursor`"
 
 static PyObject *
-psyco_conn_cursor(connectionObject *self, PyObject *args, PyObject *keywds)
+psyco_conn_cursor(connectionObject *self, PyObject *args, PyObject *kwargs)
 {
-    const char *name = NULL;
-    PyObject *obj, *factory = NULL, *withhold = NULL;
+    PyObject *obj = NULL;
+    PyObject *rv = NULL;
+    PyObject *name = Py_None;
+    PyObject *factory = Py_None;
+    PyObject *withhold = Py_False;
+    PyObject *scrollable = Py_None;
 
-    static char *kwlist[] = {"name", "cursor_factory", "withhold", NULL};
+    static char *kwlist[] = {
+        "name", "cursor_factory", "withhold", "scrollable", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|sOO", kwlist,
-                                     &name, &factory, &withhold)) {
-        return NULL;
+    EXC_IF_CONN_CLOSED(self);
+
+    if (!PyArg_ParseTupleAndKeywords(
+            args, kwargs, "|OOOO", kwlist,
+            &name, &factory, &withhold, &scrollable)) {
+        goto exit;
     }
 
-    if (withhold != NULL) {
-        if (PyObject_IsTrue(withhold) && name == NULL) {
-            PyErr_SetString(ProgrammingError,
-                "'withhold=True can be specified only for named cursors");
-            return NULL;
+    if (factory == Py_None) {
+        if (self->cursor_factory && self->cursor_factory != Py_None) {
+            factory = self->cursor_factory;
+        }
+        else {
+            factory = (PyObject *)&cursorType;
         }
     }
-    
-    EXC_IF_CONN_CLOSED(self);
 
     if (self->status != CONN_STATUS_READY &&
         self->status != CONN_STATUS_BEGIN &&
         self->status != CONN_STATUS_PREPARED) {
         PyErr_SetString(OperationalError,
                         "asynchronous connection attempt underway");
-        return NULL;
+        goto exit;
     }
 
-    if (name != NULL && self->async == 1) {
+    if (name != Py_None && self->async == 1) {
         PyErr_SetString(ProgrammingError,
                         "asynchronous connections "
                         "cannot produce named cursors");
-        return NULL;
+        goto exit;
     }
 
-    Dprintf("psyco_conn_cursor: new cursor for connection at %p", self);
-    Dprintf("psyco_conn_cursor:     parameters: name = %s", name);
+    Dprintf("psyco_conn_cursor: new %s cursor for connection at %p",
+        (name == Py_None ? "unnamed" : "named"), self);
 
-    if (factory == NULL) factory = (PyObject *)&cursorType;
-    if (name)
-        obj = PyObject_CallFunction(factory, "Os", self, name);
-    else
-        obj = PyObject_CallFunctionObjArgs(factory, self, NULL);
+    if (!(obj = PyObject_CallFunctionObjArgs(factory, self, name, NULL))) {
+        goto exit;
+    }
 
-    if (obj == NULL) return NULL;
     if (PyObject_IsInstance(obj, (PyObject *)&cursorType) == 0) {
         PyErr_SetString(PyExc_TypeError,
-            "cursor factory must be subclass of psycopg2._psycopg.cursor");
-        Py_DECREF(obj);
-        return NULL;
+            "cursor factory must be subclass of psycopg2.extensions.cursor");
+        goto exit;
+    }
+
+    if (0 != psyco_curs_withhold_set((cursorObject *)obj, withhold)) {
+        goto exit;
+    }
+    if (0 != psyco_curs_scrollable_set((cursorObject *)obj, scrollable)) {
+        goto exit;
     }
-    
-    if (withhold != NULL && PyObject_IsTrue(withhold))
-        ((cursorObject*)obj)->withhold = 1;
 
     Dprintf("psyco_conn_cursor: new cursor at %p: refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
         obj, Py_REFCNT(obj)
-      );
-    return obj;
+    );
+
+    rv = obj;
+    obj = NULL;
+
+exit:
+    Py_XDECREF(obj);
+    return rv;
 }
 
 
@@ -120,16 +133,13 @@ psyco_conn_cursor(connectionObject *self, PyObject *args, PyObject *keywds)
 #define psyco_conn_close_doc "close() -- Close the connection."
 
 static PyObject *
-psyco_conn_close(connectionObject *self, PyObject *args)
+psyco_conn_close(connectionObject *self)
 {
-    EXC_IF_CONN_CLOSED(self);
-
     Dprintf("psyco_conn_close: closing connection at %p", self);
     conn_close(self);
     Dprintf("psyco_conn_close: connection at %p closed", self);
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -138,7 +148,7 @@ psyco_conn_close(connectionObject *self, PyObject *args)
 #define psyco_conn_commit_doc "commit() -- Commit all changes to database."
 
 static PyObject *
-psyco_conn_commit(connectionObject *self, PyObject *args)
+psyco_conn_commit(connectionObject *self)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, commit);
@@ -147,8 +157,7 @@ psyco_conn_commit(connectionObject *self, PyObject *args)
     if (conn_commit(self) < 0)
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -158,7 +167,7 @@ psyco_conn_commit(connectionObject *self, PyObject *args)
 "rollback() -- Roll back all changes done to database."
 
 static PyObject *
-psyco_conn_rollback(connectionObject *self, PyObject *args)
+psyco_conn_rollback(connectionObject *self)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, rollback);
@@ -167,8 +176,7 @@ psyco_conn_rollback(connectionObject *self, PyObject *args)
     if (conn_rollback(self) < 0)
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -181,7 +189,7 @@ psyco_conn_xid(connectionObject *self, PyObject *args, PyObject *kwargs)
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_TPC_NOT_SUPPORTED(self);
 
-    return PyObject_Call((PyObject *)&XidType, args, kwargs);
+    return PyObject_Call((PyObject *)&xidType, args, kwargs);
 }
 
 
@@ -192,7 +200,7 @@ static PyObject *
 psyco_conn_tpc_begin(connectionObject *self, PyObject *args)
 {
     PyObject *rv = NULL;
-    XidObject *xid = NULL;
+    xidObject *xid = NULL;
     PyObject *oxid;
 
     EXC_IF_CONN_CLOSED(self);
@@ -232,7 +240,7 @@ exit:
 "tpc_prepare() -- perform the first phase of a two-phase transaction."
 
 static PyObject *
-psyco_conn_tpc_prepare(connectionObject *self, PyObject *args)
+psyco_conn_tpc_prepare(connectionObject *self)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, tpc_prepare);
@@ -252,8 +260,7 @@ psyco_conn_tpc_prepare(connectionObject *self, PyObject *args)
      * can be performed until commit. */
     self->status = CONN_STATUS_PREPARED;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -284,7 +291,7 @@ _psyco_conn_tpc_finish(connectionObject *self, PyObject *args,
     _finish_f opc_f, char *tpc_cmd)
 {
     PyObject *oxid = NULL;
-    XidObject *xid = NULL;
+    xidObject *xid = NULL;
     PyObject *rv = NULL;
 
     if (!PyArg_ParseTuple(args, "|O", &oxid)) { goto exit; }
@@ -376,7 +383,7 @@ psyco_conn_tpc_rollback(connectionObject *self, PyObject *args)
 "tpc_recover() -- returns a list of pending transaction IDs."
 
 static PyObject *
-psyco_conn_tpc_recover(connectionObject *self, PyObject *args)
+psyco_conn_tpc_recover(connectionObject *self)
 {
     EXC_IF_CONN_CLOSED(self);
     EXC_IF_CONN_ASYNC(self, tpc_recover);
@@ -387,7 +394,52 @@ psyco_conn_tpc_recover(connectionObject *self, PyObject *args)
 }
 
 
-#ifdef PSYCOPG_EXTENSIONS
+#define psyco_conn_enter_doc \
+"__enter__ -> self"
+
+static PyObject *
+psyco_conn_enter(connectionObject *self)
+{
+    EXC_IF_CONN_CLOSED(self);
+
+    Py_INCREF(self);
+    return (PyObject *)self;
+}
+
+
+#define psyco_conn_exit_doc \
+"__exit__ -- commit if no exception, else roll back"
+
+static PyObject *
+psyco_conn_exit(connectionObject *self, PyObject *args)
+{
+    PyObject *type, *name, *tb;
+    PyObject *tmp = NULL;
+    PyObject *rv = NULL;
+
+    if (!PyArg_ParseTuple(args, "OOO", &type, &name, &tb)) {
+        goto exit;
+    }
+
+    if (type == Py_None) {
+        if (!(tmp = PyObject_CallMethod((PyObject *)self, "commit", NULL))) {
+            goto exit;
+        }
+    } else {
+        if (!(tmp = PyObject_CallMethod((PyObject *)self, "rollback", NULL))) {
+            goto exit;
+        }
+    }
+
+    /* success (of the commit or rollback, there may have been an exception in
+     * the block). Return None to avoid swallowing the exception */
+    rv = Py_None;
+    Py_INCREF(rv);
+
+exit:
+    Py_XDECREF(tmp);
+    return rv;
+}
 
 
 /* parse a python object into one of the possible isolation level values */
@@ -528,13 +580,12 @@ psyco_conn_set_session(connectionObject *self, PyObject *args, PyObject *kwargs)
         if (-1 == c_autocommit) { return NULL; }
     }
 
-    if (0 != conn_set_session(self,
+    if (0 > conn_set_session(self,
             c_isolevel, c_readonly, c_deferrable, c_autocommit)) {
         return NULL;
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -550,7 +601,7 @@ psyco_conn_autocommit_get(connectionObject *self)
     return ret;
 }
 
-static PyObject *
+BORROWED static PyObject *
 _psyco_conn_autocommit_set_checks(connectionObject *self)
 {
     /* wrapper to use the EXC_IF macros.
@@ -616,8 +667,7 @@ psyco_conn_set_isolation_level(connectionObject *self, PyObject *args)
         return NULL;
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 /* set_client_encoding method - set client encoding */
@@ -637,7 +687,7 @@ psyco_conn_set_client_encoding(connectionObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "s", &enc)) return NULL;
 
-    if (conn_set_client_encoding(self, enc) == 0) {
+    if (conn_set_client_encoding(self, enc) >= 0) {
         Py_INCREF(Py_None);
         rv = Py_None;
     }
@@ -650,10 +700,8 @@ psyco_conn_set_client_encoding(connectionObject *self, PyObject *args)
 "get_transaction_status() -- Get backend transaction status."
 
 static PyObject *
-psyco_conn_get_transaction_status(connectionObject *self, PyObject *args)
+psyco_conn_get_transaction_status(connectionObject *self)
 {
-    EXC_IF_CONN_CLOSED(self);
-
     return PyInt_FromLong((long)PQtransactionStatus(self->pgconn));
 }
 
@@ -680,8 +728,7 @@ psyco_conn_get_parameter_status(connectionObject *self, PyObject *args)
 
     val = PQparameterStatus(self->pgconn, param);
     if (!val) {
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
     return conn_text_from_chars(self, val);
 }
@@ -701,8 +748,8 @@ psyco_conn_get_parameter_status(connectionObject *self, PyObject *args)
 static PyObject *
 psyco_conn_lobject(connectionObject *self, PyObject *args, PyObject *keywds)
 {
-    Oid oid=InvalidOid, new_oid=InvalidOid;
-    char *new_file = NULL;
+    Oid oid = InvalidOid, new_oid = InvalidOid;
+    const char *new_file = NULL;
     const char *smode = "";
     PyObject *factory = (PyObject *)&lobjectType;
     PyObject *obj;
@@ -710,7 +757,7 @@ psyco_conn_lobject(connectionObject *self, PyObject *args, PyObject *keywds)
     static char *kwlist[] = {"oid", "mode", "new_oid", "new_file",
                              "cursor_factory", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|izizO", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|IzIzO", kwlist,
                                      &oid, &smode, &new_oid, &new_file,
                                      &factory)) {
         return NULL;
@@ -722,22 +769,22 @@ psyco_conn_lobject(connectionObject *self, PyObject *args, PyObject *keywds)
     EXC_IF_TPC_PREPARED(self, lobject);
 
     Dprintf("psyco_conn_lobject: new lobject for connection at %p", self);
-    Dprintf("psyco_conn_lobject:     parameters: oid = %d, mode = %s",
+    Dprintf("psyco_conn_lobject:     parameters: oid = %u, mode = %s",
             oid, smode);
     Dprintf("psyco_conn_lobject:     parameters: new_oid = %d, new_file = %s",
             new_oid, new_file);
 
     if (new_file)
-        obj = PyObject_CallFunction(factory, "Oisis",
+        obj = PyObject_CallFunction(factory, "OIsIs",
             self, oid, smode, new_oid, new_file);
     else
-        obj = PyObject_CallFunction(factory, "Oisi",
+        obj = PyObject_CallFunction(factory, "OIsI",
             self, oid, smode, new_oid);
 
     if (obj == NULL) return NULL;
     if (PyObject_IsInstance(obj, (PyObject *)&lobjectType) == 0) {
         PyErr_SetString(PyExc_TypeError,
-            "lobject factory must be subclass of psycopg2._psycopg.lobject");
+            "lobject factory must be subclass of psycopg2.extensions.lobject");
         Py_DECREF(obj);
         return NULL;
     }
@@ -781,8 +828,7 @@ psyco_conn_reset(connectionObject *self)
     if (res < 0)
         return NULL;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 static PyObject *
@@ -794,6 +840,10 @@ psyco_conn_get_exception(PyObject *self, void *closure)
     return exception;
 }
 
+
+#define psyco_conn_poll_doc \
+"poll() -> int -- Advance the connection or query process without blocking."
+
 static PyObject *
 psyco_conn_poll(connectionObject *self)
 {
@@ -811,8 +861,6 @@ psyco_conn_poll(connectionObject *self)
 }
 
 
-/* extension: fileno - return the file descriptor of the connection */
-
 #define psyco_conn_fileno_doc \
 "fileno() -> int -- Return file descriptor associated to database connection."
 
@@ -829,8 +877,6 @@ psyco_conn_fileno(connectionObject *self)
 }
 
 
-/* extension: isexecuting - check for asynchronous operations */
-
 #define psyco_conn_isexecuting_doc                           \
 "isexecuting() -> bool -- Return True if the connection is " \
  "executing an asynchronous operation."
@@ -862,8 +908,6 @@ psyco_conn_isexecuting(connectionObject *self)
 }
 
 
-/* extension: cancel - cancel the current operation */
-
 #define psyco_conn_cancel_doc                           \
 "cancel() -- cancel the current operation"
 
@@ -889,12 +933,9 @@ psyco_conn_cancel(connectionObject *self)
         PyErr_SetString(OperationalError, errbuf);
         return NULL;
     }
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
-#endif  /* PSYCOPG_EXTENSIONS */
-
 
 /** the connection object **/
 
@@ -922,7 +963,10 @@ static struct PyMethodDef connectionObject_methods[] = {
      METH_VARARGS, psyco_conn_tpc_rollback_doc},
     {"tpc_recover", (PyCFunction)psyco_conn_tpc_recover,
      METH_NOARGS, psyco_conn_tpc_recover_doc},
-#ifdef PSYCOPG_EXTENSIONS
+    {"__enter__", (PyCFunction)psyco_conn_enter,
+     METH_NOARGS, psyco_conn_enter_doc},
+    {"__exit__", (PyCFunction)psyco_conn_exit,
+     METH_VARARGS, psyco_conn_exit_doc},
     {"set_session", (PyCFunction)psyco_conn_set_session,
      METH_VARARGS|METH_KEYWORDS, psyco_conn_set_session_doc},
     {"set_isolation_level", (PyCFunction)psyco_conn_set_isolation_level,
@@ -940,21 +984,19 @@ static struct PyMethodDef connectionObject_methods[] = {
     {"reset", (PyCFunction)psyco_conn_reset,
      METH_NOARGS, psyco_conn_reset_doc},
     {"poll", (PyCFunction)psyco_conn_poll,
-     METH_NOARGS, psyco_conn_lobject_doc},
+     METH_NOARGS, psyco_conn_poll_doc},
     {"fileno", (PyCFunction)psyco_conn_fileno,
      METH_NOARGS, psyco_conn_fileno_doc},
     {"isexecuting", (PyCFunction)psyco_conn_isexecuting,
      METH_NOARGS, psyco_conn_isexecuting_doc},
     {"cancel", (PyCFunction)psyco_conn_cancel,
      METH_NOARGS, psyco_conn_cancel_doc},
-#endif
     {NULL}
 };
 
 /* object member list */
 
 static struct PyMemberDef connectionObject_members[] = {
-#ifdef PSYCOPG_EXTENSIONS
     {"closed", T_LONG, offsetof(connectionObject, closed), READONLY,
         "True if the connection is closed."},
     {"encoding", T_STRING, offsetof(connectionObject, encoding), READONLY,
@@ -968,6 +1010,8 @@ static struct PyMemberDef connectionObject_members[] = {
     {"status", T_INT,
         offsetof(connectionObject, status), READONLY,
         "The current transaction status."},
+    {"cursor_factory", T_OBJECT, offsetof(connectionObject, cursor_factory), 0,
+        "Default cursor_factory for cursor()."},
     {"string_types", T_OBJECT, offsetof(connectionObject, string_types), READONLY,
         "A set of typecasters to convert textual values."},
     {"binary_types", T_OBJECT, offsetof(connectionObject, binary_types), READONLY,
@@ -978,7 +1022,6 @@ static struct PyMemberDef connectionObject_members[] = {
     {"server_version", T_INT,
         offsetof(connectionObject, server_version), READONLY,
         "Server version."},
-#endif
     {NULL}
 };
 
@@ -986,7 +1029,6 @@ static struct PyMemberDef connectionObject_members[] = {
     { #exc, psyco_conn_get_exception, NULL, exc ## _doc, &exc }
 
 static struct PyGetSetDef connectionObject_getsets[] = {
-    /* DBAPI-2.0 extensions (exception objects) */
     EXCEPTION_GETTER(Error),
     EXCEPTION_GETTER(Warning),
     EXCEPTION_GETTER(InterfaceError),
@@ -997,7 +1039,6 @@ static struct PyGetSetDef connectionObject_getsets[] = {
     EXCEPTION_GETTER(IntegrityError),
     EXCEPTION_GETTER(DataError),
     EXCEPTION_GETTER(NotSupportedError),
-#ifdef PSYCOPG_EXTENSIONS
     { "autocommit",
         (getter)psyco_conn_autocommit_get,
         (setter)psyco_conn_autocommit_set,
@@ -1006,7 +1047,6 @@ static struct PyGetSetDef connectionObject_getsets[] = {
         (getter)psyco_conn_isolation_level_get,
         (setter)NULL,
         "The current isolation level." },
-#endif
     {NULL}
 };
 #undef EXCEPTION_GETTER
@@ -1024,10 +1064,7 @@ connection_setup(connectionObject *self, const char *dsn, long int async)
             self, async, Py_REFCNT(self)
       );
 
-    if (!(self->dsn = strdup(dsn))) {
-        PyErr_NoMemory();
-        goto exit;
-    }
+    if (0 > psycopg_strdup(&self->dsn, dsn, 0)) { goto exit; }
     if (!(self->notice_list = PyList_New(0))) { goto exit; }
     if (!(self->notifies = PyList_New(0))) { goto exit; }
     self->async = async;
@@ -1051,6 +1088,7 @@ connection_setup(connectionObject *self, const char *dsn, long int async)
         res = 0;
     }
 
+exit:
     /* here we obfuscate the password even if there was a connection error */
     pos = strstr(self->dsn, "password");
     if (pos != NULL) {
@@ -1058,30 +1096,12 @@ connection_setup(connectionObject *self, const char *dsn, long int async)
             *pos = 'x';
     }
 
-exit:
     return res;
 }
 
-static void
-connection_dealloc(PyObject* obj)
+static int
+connection_clear(connectionObject *self)
 {
-    connectionObject *self = (connectionObject *)obj;
-
-    if (self->weakreflist) {
-        PyObject_ClearWeakRefs(obj);
-    }
-
-    PyObject_GC_UnTrack(self);
-
-    if (self->closed == 0) conn_close(self);
-
-    conn_notice_clean(self);
-
-    if (self->dsn) free(self->dsn);
-    PyMem_Free(self->encoding);
-    PyMem_Free(self->codec);
-    if (self->critical) free(self->critical);
-
     Py_CLEAR(self->tpc_xid);
     Py_CLEAR(self->async_cursor);
     Py_CLEAR(self->notice_list);
@@ -1089,6 +1109,34 @@ connection_dealloc(PyObject* obj)
     Py_CLEAR(self->notifies);
     Py_CLEAR(self->string_types);
     Py_CLEAR(self->binary_types);
+    return 0;
+}
+
+static void
+connection_dealloc(PyObject* obj)
+{
+    connectionObject *self = (connectionObject *)obj;
+
+    /* Make sure to untrack the connection before calling conn_close, which may
+     * allow a different thread to try and dealloc the connection again,
+     * resulting in a double-free segfault (ticket #166). */
+    PyObject_GC_UnTrack(self);
+
+    conn_close(self);
+
+    if (self->weakreflist) {
+        PyObject_ClearWeakRefs(obj);
+    }
+
+    conn_notice_clean(self);
+
+    PyMem_Free(self->dsn);
+    PyMem_Free(self->encoding);
+    PyMem_Free(self->codec);
+    if (self->critical) free(self->critical);
+    if (self->cancel) PQfreeCancel(self->cancel);
+
+    connection_clear(self);
 
     pthread_mutex_destroy(&(self->lock));
 
@@ -1119,18 +1167,12 @@ connection_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     return type->tp_alloc(type, 0);
 }
 
-static void
-connection_del(PyObject* self)
-{
-    PyObject_GC_Del(self);
-}
-
 static PyObject *
 connection_repr(connectionObject *self)
 {
     return PyString_FromFormat(
         "<connection object at %p; dsn: '%s', closed: %ld>",
-        self, self->dsn, self->closed);
+        self, (self->dsn ? self->dsn : "<unintialized>"), self->closed);
 }
 
 static int
@@ -1158,9 +1200,8 @@ connection_traverse(connectionObject *self, visitproc visit, void *arg)
 
 PyTypeObject connectionType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "psycopg2._psycopg.connection",
-    sizeof(connectionObject),
-    0,
+    "psycopg2.extensions.connection",
+    sizeof(connectionObject), 0,
     connection_dealloc, /*tp_dealloc*/
     0,          /*tp_print*/
     0,          /*tp_getattr*/
@@ -1171,47 +1212,30 @@ PyTypeObject connectionType = {
     0,          /*tp_as_sequence*/
     0,          /*tp_as_mapping*/
     0,          /*tp_hash */
-
     0,          /*tp_call*/
     (reprfunc)connection_repr, /*tp_str*/
     0,          /*tp_getattro*/
     0,          /*tp_setattro*/
     0,          /*tp_as_buffer*/
-
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC |
         Py_TPFLAGS_HAVE_WEAKREFS,
                 /*tp_flags*/
     connectionType_doc, /*tp_doc*/
-
     (traverseproc)connection_traverse, /*tp_traverse*/
-    0,          /*tp_clear*/
-
+    (inquiry)connection_clear, /*tp_clear*/
     0,          /*tp_richcompare*/
     offsetof(connectionObject, weakreflist), /* tp_weaklistoffset */
-
     0,          /*tp_iter*/
     0,          /*tp_iternext*/
-
-    /* Attribute descriptor and subclassing stuff */
-
     connectionObject_methods, /*tp_methods*/
     connectionObject_members, /*tp_members*/
     connectionObject_getsets, /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
-
     0,          /*tp_descr_get*/
     0,          /*tp_descr_set*/
     0,          /*tp_dictoffset*/
-
     connection_init, /*tp_init*/
-    0, /*tp_alloc  will be set to PyType_GenericAlloc in module init*/
+    0,          /*tp_alloc*/
     connection_new, /*tp_new*/
-    (freefunc)connection_del, /*tp_free  Low-level free-memory routine */
-    0,          /*tp_is_gc For PyObject_IS_GC */
-    0,          /*tp_bases*/
-    0,          /*tp_mro method resolution order */
-    0,          /*tp_cache*/
-    0,          /*tp_subclasses*/
-    0           /*tp_weaklist*/
 };
